/*
 * Programm zum Berechnen der Anstellwinkel der Rotorbltter im Impeller
 * 
 * Abgeleitet von SUN's Java Beispielen:  @(#)ArcTest.java von 1.4 97/02/05
 *
 * name:	impeller.java
 * author:	sven-volker Nowarra, peb.nowarra@t-online.de
 * date:		may 2002
 * version:	0.9
 *
 * REM:
 * Berechnung der Anstellwinkel fr die Rotor- and Statorschaufeln in einem Modell-Impeller
 * Literatur:
 * FMT Fachbuch - Das Elektro-Impeller Buch von Dirk Juras
 *
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.Toolkit;


public class Impeller extends Applet 
   {
   ImpControls controls;
   
   public void init() 
      {
      setLayout(new GridLayout(1,2));

	ImpCanvas c = new ImpCanvas();
	add(c);
	add(controls = new ImpControls(c));
      }

   public void start()
      {
	controls.setEnabled(true);
      }

   public void stop() 
      {
	controls.setEnabled(false);
      }

   public boolean handleEvent(Event e)
      {
	if (e.id == Event.WINDOW_DESTROY) 
          {
	    System.exit(0);
          }
      return false;
      }

   public static void main(String args[])
      {
      Frame f = new Frame("Impeller");
      Impeller Impeller = new Impeller();

      Impeller.init();
      Impeller.start();

      f.add("Center", Impeller);
      f.setSize(300, 300);
      f.show();
      }
   }
    

class ImpCanvas extends Canvas
   {
   private int a,b,c;			// Zwischenvariablen im Dreieck
   int	i_ri = 10;
   int 	i_ra = 20;
   double   d_ua = 0;
   double   d_ui = 0;
   double   d_alpha_i = 0;
   double   d_alpha_a = 0;
   int         d_stator_i = 0;
   int         d_stator_a = 0;

   // boolean	filled = false;
   Font	font;

   public void paint(Graphics g) 
      {
      this.setSize(300,300);
      Rectangle r = getBounds();
      // System.out.println("r.height = " + r.height);
      // System.out.println("r.width = " + r.width);
      int hlines = r.height / 10;
      int vlines = r.width / 10;
      int bauwinkel_i, bauwinkel_a;

      // text Position in der Grafik, und Offsets fuer die Positionierung
      int text_xpos = 0;
      int text_ypos = r.height;
      int txt_line_height = 12; 		// Offset zwischen zwei Text Zeilen

      int txt_offset_x_vu = 70;		// Text Umfangsgeschw. Offset nach rechts
      int txt_offset_y_vu = 12;		// Text Umfangsgeschw. Offset nach unten

      int txt_offset_x_rotor = 140;		// Text Rotor Offset nach rechts

      int txt_offset_x_stator = 10; 		// Text Stator Offset nach rechts


      // paint some horizontal and vertical lines ...
      g.setColor(Color.pink);
      for (int i = 1; i <= hlines; i++) 
         {
         g.drawLine(0, i * 10, r.width, i * 10);
         }
      for (int i = 1; i <= vlines; i++)
         {
         g.drawLine(i * 10, 0, i * 10, r.height);
          }

      // we draw some coordinate lines
      g.setColor(Color.black);
      g.setFont(font);
      // horizontal zero line (x-axis)
      g.drawLine(1, r.height-1, r.width-1, r.height-1);
      // vertical zero line (y-axis)
      g.drawLine(0, 0, 0, r.height);
      // noch den text oben rechts ausgeben ...
      g.setColor(Color.blue);
      g.drawString("Umfangsgeschw. aussen = " + (int)Math.floor(d_ua) + " m/sec)", text_xpos + txt_offset_x_vu, txt_offset_y_vu); 
      g.drawString("Umfangsgeschw. innen  = " + (int)Math.floor(d_ui) + " m/sec)", text_xpos + txt_offset_x_vu, txt_offset_y_vu + txt_line_height);
      // und die Linien malen ...
      g.drawString("Baubrettwinkel (gerundet):", text_xpos + txt_offset_x_rotor, text_ypos - (3 * txt_line_height)); 

      // Rotorblatt innen
      g.setColor(Color.green);
      bauwinkel_i = 90 - (int)Math.floor(d_alpha_i);
      // System.out.println("Bauwinkel innen = " + bauwinkel_i);
      g.drawArc(0 - r.width - 1, 0, r.width * 2, r.height * 2, 0, bauwinkel_i);
      g.drawString("Innenwinkel = " + bauwinkel_i, text_xpos + txt_offset_x_rotor, text_ypos - (2 * txt_line_height));
      c = r.width;
      // System.out.println("c_innen = " + c);
      a = (int)Math.floor(Math.sin(bauwinkel_i * Math.PI / 180) * c);
      // System.out.println("a_innen = " + a);
      b = (int)Math.floor(Math.sqrt((c * c) - (a * a)));
      // System.out.println("b_innen = " + b);
      // System.out.println("a_innen nach Korrektur = " + a);
      // links, unten, von_rechts, von_oben
      g.drawLine(1, r.height, b, (r.height - a));

      // Statorblatt Innen
      g.drawString("Stator Innenwinkel = " + d_stator_i, text_xpos + txt_offset_x_stator, (text_ypos / 2) + txt_line_height);
      g.drawArc(0 - (r.width / 2), 0 - (r.height / 2), r.width, r.height, 270, d_stator_i);
      a = (int)Math.floor(Math.sin(d_stator_i * Math.PI / 180) * r.height / 2);
      b = (int)Math.floor(Math.sqrt((r.width / 2 * r.width / 2) - (a * a)));
      g.drawLine(0, 0, a, b);

      // Rotorblatt aussen
      g.setColor(Color.red);
      bauwinkel_a = 90 - (int)Math.floor(d_alpha_a);
      g.drawArc(0 - r.width - 2, 0, r.width * 2, r.height * 2, 0, bauwinkel_a);
      g.drawString("Aussenwinkel = " + bauwinkel_a, text_xpos + txt_offset_x_rotor, text_ypos - txt_line_height); 
      a = (int)Math.floor(Math.sin(bauwinkel_a * Math.PI / 180) * r.width);
      b = (int)Math.floor(Math.sqrt((r.width * r.width) - (a * a)));
      g.drawLine(1, r.height - 1, b, (r.height - a));

      // Statorblatt aussen
      g.drawString("Stator Aussenwinkel = " + d_stator_a, text_xpos + txt_offset_x_stator, (text_ypos / 2) + (2 * txt_line_height));
      g.drawArc(0 - (r.width / 2), 0 - (r.height / 2), r.width, r.height, 270, d_stator_a);
      a = (int)Math.floor(Math.sin(d_stator_a * Math.PI / 180) * r.height / 2);
      b = (int)Math.floor(Math.sqrt((r.width / 2 * r.width / 2) - (a * a)));
      g.drawLine(0, 0, a, b);

      g.setColor(Color.blue);
      // g.drawString("Die Impellerwinkel aussen/innen sind noch vertauscht - oder ?", text_xpos, text_ypos/2); 
      // System.out.println("---");
      }

   public void redraw(double ui, double ua, double alpha_i, double alpha_a, double stator_i,double stator_a)
      {
      this.d_ui = ui;
      this.d_ua = ua;
      this.d_alpha_i = alpha_i;
      this.d_alpha_a = alpha_a;
      this.d_stator_i = (int)Math.floor(stator_i);
      this.d_stator_a = (int)Math.floor(stator_a);
      repaint();
      }
   }


class ImpControls extends Panel implements ActionListener
   {
   TextField ml;	// Motorleistung
   TextField si;	// gewnschter Schub
   TextField sg;	// gewnschte Schubgeschwindigkeit
   TextField mg;	// Flugmodellgeschwindigkeit
   TextField rpm;	// Motordrehzahl (in rpm)
   TextField ri;	// Durchmesser innen
   TextField ra;	// Durchmesser aussen
   TextField dd;	// Dsenverjngung in Prozent

   ImpCanvas canvas;

   void buildConstraints(GridBagConstraints c, int gx, int gy, int gw, int gh, int wx, int wy) 
      {
      c.gridx = gx;
      c.gridy = gy;
      c.gridwidth = gw;
      c.gridheight = gh;
      c.weightx = wx;
      c.weighty = wy;
      } 

   public ImpControls(ImpCanvas canvas) 
      {
      this.canvas = canvas;

      GridBagLayout gridbag = new GridBagLayout();
      GridBagConstraints gbc = new GridBagConstraints();

      Button b;
      setFont(new Font("Helvetica", Font.PLAIN, 10));
      setLayout(gridbag); 
      gbc.fill = GridBagConstraints.NONE;

      // Der Label "Abgabeleistung"
      buildConstraints(gbc, 0, 0, 1, 1, 0, 0);
      gbc.anchor = GridBagConstraints.WEST;
      Label label1 = new Label("Abgabeleistung (in Watt): ");
      // label1.setAlignment(Label.LEFT);
      gridbag.setConstraints(label1, gbc); 
      add(label1);

      // Textfeld "Abgabeleistung"
      buildConstraints(gbc, 1, 0, 1, 1, 0, 0);
      gbc.anchor = GridBagConstraints.EAST;
      ml = new TextField("420",4);
      gridbag.setConstraints(ml, gbc);
      add(ml);

      // Der Label "Schub"
      buildConstraints(gbc, 0, 1, 1, 1, 0, 0);
      gbc.anchor = GridBagConstraints.WEST;
      Label label2 = new Label("gewnschter Schub (in N): ");
      // label2.setAlignment(Label.LEFT);
      gridbag.setConstraints(label2, gbc); 
      add(label2);

      // Textfeld "Schub"
      buildConstraints(gbc, 1, 1, 1, 1, 0, 0);
      gbc.anchor = GridBagConstraints.EAST;
      si = new TextField("10",4);
      gridbag.setConstraints(si, gbc);
      add(si);

      // Der Label "gewnschte Schubgeschw. (in m/sec): "
      buildConstraints(gbc, 0, 2, 1, 1, 0, 0);
      gbc.anchor = GridBagConstraints.WEST;
      Label label3 = new Label("gew. Schubgeschw. (in m/sec): ");
      gridbag.setConstraints(label3, gbc); 
      add(label3);

      // Textfeld "Schubgeschw. (in m/sec)"
      buildConstraints(gbc, 1, 2, 1, 1, 0, 0);
      gbc.anchor = GridBagConstraints.EAST;
      sg = new TextField("55",4);
      gridbag.setConstraints(sg, gbc);
      add(sg);

      // Der Zwischenlabel 
      buildConstraints(gbc, 0, 3, 2, 1, 0, 0);
      gbc.anchor = GridBagConstraints.WEST;
      Label label9 = new Label("-----------------");
      gridbag.setConstraints(label9, gbc); 
      add(label9);

      // Der Label "Flugmodellgeschw. (in m/sec)"
      buildConstraints(gbc, 0, 4, 1, 1, 0, 0);
      gbc.anchor = GridBagConstraints.WEST;
      Label label4 = new Label("Flugmodellgeschw. (in m/sec): ");
      gridbag.setConstraints(label4, gbc); 
      add(label4);

      // Textfeld "Flugmodellgeschw. (in m/sec)"
      buildConstraints(gbc, 1, 4, 1, 1, 0, 0);
      gbc.anchor = GridBagConstraints.EAST;
      // c.gridwidth = GridBaggbc.REMAINDER; //end of row
      mg = new TextField("30",4);
      gridbag.setConstraints(mg, gbc);
      add(mg);

       // Der Label "Motordrehzahl (in rpm)"
      buildConstraints(gbc, 0, 5, 1, 1, 0, 0);
      gbc.anchor = GridBagConstraints.WEST;
      Label label5 = new Label("Motordrehzahl (in rpm): ");
      gridbag.setConstraints(label5, gbc); 
      add(label5);

       // Textfeld "Motordrehzahl (in rpm)"
      buildConstraints(gbc, 1, 5, 1, 1, 0, 0);
      gbc.anchor = GridBagConstraints.EAST;
      rpm = new TextField("20000",4);
      gridbag.setConstraints(rpm, gbc);
      add(rpm);
 
      // Der Label "Aussendurchmesser (in mm): "
      buildConstraints(gbc, 0, 6, 1, 1, 0, 0);
      gbc.anchor = GridBagConstraints.WEST;
      Label label6 = new Label("Aussendurchmesser (in mm): ");
      gridbag.setConstraints(label6, gbc); 
      add(label6);

       // Textfeld "Aussendurchmesser (in mm)"
      buildConstraints(gbc, 1, 6, 1, 1, 0, 0);
      gbc.anchor = GridBagConstraints.EAST;
      ra = new TextField("80",4);
      gridbag.setConstraints(ra, gbc);
      add(ra);

      // Der Label "Innendurchmesser (in mm)"
      buildConstraints(gbc, 0, 7, 1, 1, 0, 0);
      gbc.anchor = GridBagConstraints.WEST;
      Label label7 = new Label("Innendurchmesser (in mm): ", Label.LEFT);
      // label7.setAlignment(Label.LEFT);
      gridbag.setConstraints(label7, gbc); 
      add(label7);

       // Textfeld "Innenndurchmesser (in mm)"
      buildConstraints(gbc, 1, 7, 1, 1, 0, 0);
      gbc.anchor = GridBagConstraints.EAST;
      ri = new TextField("40",4);
      gridbag.setConstraints(ri, gbc);
      add(ri);

      // Der Label "Dsenverjngung (in %)"
      buildConstraints(gbc, 0, 8, 1, 1, 0, 0);
      gbc.anchor = GridBagConstraints.WEST;
      Label label8 = new Label("Dsenverjngung (in %): ");
      gridbag.setConstraints(label8, gbc); 
      add(label8);

       // Textfeld "Dsenverjngung (in %)"
      buildConstraints(gbc, 1, 8, 1, 1, 0, 0);
      gbc.anchor = GridBagConstraints.EAST;
      dd = new TextField("10",4);
      gridbag.setConstraints(dd, gbc);
      add(dd);

      // Die Schaltflche
      buildConstraints(gbc, 0, 9, 2, 0, 0, 0);
      gbc.fill = GridBagConstraints.NONE;
      gbc.anchor = GridBagConstraints.CENTER;
      b = new Button("Calculate");
      gridbag.setConstraints(b, gbc);
      b.addActionListener(this);
      add(b);
      }

   public void actionPerformed(ActionEvent ev) 
      {
      String label = ev.getActionCommand();
      double u_x, u_y;
      double ua, ui;
      double v1i, v2i;			// v1i ist Modellfluggeschw. (mg), v2i der Vektor des Stators
      double w1i, w2i;
      double alpha1, alpha2;		// Hilfswinkel in den beiden Impeller-Dreiecken
      double alpha_i, alpha_a;		// Rotor Winkel
      double alpha_st_i, alpha_st_a;	// Stator Winkel

      // hier knnen wir noch einen Pieps einbauen ...
      // Toolkit.getDefaultToolkit().beep();

      // zuerst muss die Umfangsgeschwindigkeit der Rotorschaufeln
      // im Impeller berechnet werden, u = 2 * pi * r * rpm / 60
      // das entspricht: pi * d * rpm / 60
      // Achtung - wir haben die Eingaben in Millimeter, also muss durch 1000 geteilt werden !
      ui = Math.PI * (Integer.parseInt(ri.getText().trim())) * (Integer.parseInt(rpm.getText().trim())) / 60 / 1000;
      ua = Math.PI * (Integer.parseInt(ra.getText().trim())) * (Integer.parseInt(rpm.getText().trim())) / 60 / 1000;

      // mit dieser Geschwindigkeit (ui oder ua) wird normalerweise der Vektor nach rechts im 
      // rechtwinkligen Dreieick gezeichnet (Gegenkathete), und die vorgegebene 
      // Modellgeschwindigkeit (mg) wird der Vektor nach unten (Ankathete). Die Hypothenuse wird 
      // dann w1i. Es gilt also der Satz des Pythagoras a + b = c. 
      // Daraus folgt: ui * ui + mg * mg = sqr(w1i)

      v1i = Integer.parseInt(mg.getText().trim());
      w1i = Math.sqrt((ui * ui) + (v1i * v1i));
      // System.out.println(w1i);
      // System.out.println("inv sin 0.7071067 sollte 45 Grad geben");
      // System.out.println(Math.asin(0.7071067) * 180 / Math.PI);

      alpha1 = Math.asin(ui / w1i) / Math.PI * 180;
      // System.out.println(alpha1);

      // jetzt wird am Dreieck noch w2i angezeichnet, als Vektor in der Verlngerung von
      // w1i. Es gilt, dass w2i = 0,8 * w1i - warum steht nicht im Buch drin ...
      // Der Vektor der Modellgeschwindigkeit wird nach unten nochmal drangehngt 
      // (senkrechte Verlngerung). Man denkt sich an "seinem Boden" eine waagrechte Linie.
      // Man erhlt so wieder ein rechtwinkliges Dreieck (rechter Winkel rechts unten), welches 
      // so gekennzeichnet ist (Bezugswinkel sei beta ...):
      // Ankathete = Bodenlinie
      // Gegenkathete = Modellgeschwindigkeitsvektor
      // Hypothenuse = w2i
      // es gilt dann: alpha2 = 90 - sin beta = 90 - sin(v1i / w2i) 
      // Dadurch erhlt w2i einen flacheren Winkel als w1i.

      w2i = 0.8 * w1i;
      // System.out.println(w2i);
      alpha2 = 90 - Math.asin(v1i / w2i) / Math.PI * 180;
      // System.out.println(alpha2);

      // beide Winkel werden addiert, durch 2 geteilt, um den Gesamtwinkel der Profilsehne zu erhalten.
      alpha_i = (alpha1 + alpha2) / 2;

      // hier jetzt die Rechnung fuer die Statorblaetter
      // gesucht ist der Winkel zur Anstellung der Rotorblaetter. u1i ist als Vektorgroesse bekannt.
      // Aus dem Dreieck u1i, v2i und w2i muessen wir also die Hoehe des Dreiecks bestimmen, um 
      // entlang dieser Achse den Winkel abzulesen. Die Hoehe ist v1i !
      // Ich teile also u1i in zwei Teile: x und y. Dann ergibt sich ein Dreieck aus y, v2i und v1i.
      // Darin kann der Winkel bestimmt werden...
      // Hier die Rechnung fuer die Statorblaetter innen:
      // y aus ui und x berechnen:
      // Wir denken uns mit w2i und v1i ein Dreieck, wo die Laenge der Grundlinie x sei...
      u_x = Math.sqrt((w2i * w2i) - (v1i * v1i));
      u_y = ui - u_x;

      // Das Dreieck v1i, y und v2i kann nach a + b = c ausgerechnet werden. v2i ist dabei die
      // Hypothenuse.
      v2i = Math.sqrt((v1i * v1i) + (u_y * u_y));
      alpha_st_i = Math.asin(u_y / w2i) / Math.PI * 180;
      // System.out.println(alpha_st_i);


      // und jetzt nochmal das ganze fr den Rand (aussen) der Rotorschaufel:
      w1i = Math.sqrt((ua * ua) + (v1i * v1i));
      alpha1 = Math.asin(ua / w1i) / Math.PI * 180;
      w2i = 0.8 * w1i;
      alpha2 = 90 - Math.asin(v1i / w2i) / Math.PI * 180;
      alpha_a = (alpha1 + alpha2) / 2;


      // und jetzt der Stator aussen
      u_x = Math.sqrt((w2i * w2i) - (v1i * v1i));
      u_y = ua - u_x;      
      v2i = Math.sqrt((v1i * v1i) + (u_y * u_y));
      alpha_st_a = Math.asin(u_y / w2i) / Math.PI * 180;
      // System.out.println(alpha_st_a);
     
//       canvas.redraw(87.1, 43.2, 48.7, 30.5);
      canvas.redraw(ui, ua, alpha_i,alpha_a,alpha_st_i,alpha_st_a);
      }

   }

